@@ -1,6 +1,6 @@
 /* _NVRM_COPYRIGHT_BEGIN_
  *
- * Copyright 2001-2018 by NVIDIA Corporation.  All rights reserved.  All
+ * Copyright 2001-2019 by NVIDIA Corporation.  All rights reserved.  All
  * information contained herein is proprietary and confidential to NVIDIA
  * Corporation.  Any use, reproduction, or disclosure without the written
  * permission of NVIDIA Corporation is prohibited.
@@ -146,16 +146,10 @@
  * is permitted to change its cred structure. Therefore, current_euid()
  * does not require the RCU's read lock on current->cred.
  */
-#if defined(NV_TASK_STRUCT_HAS_CRED)
 #define NV_CURRENT_EUID() (__kuid_val(current->cred->euid))
-#else
-#define NV_CURRENT_EUID() (__kuid_val(current->euid))
-#endif
 
 #if !defined(NV_KUID_T_PRESENT)
-typedef uid_t kuid_t;
-
-static inline uid_t __kuid_val(kuid_t uid)
+static inline uid_t __kuid_val(uid_t uid)
 {
     return uid;
 }
@@ -166,11 +160,6 @@ static inline uid_t __kuid_val(kuid_t uid)
 #include <linux/ioctl32.h>          /* register_ioctl32_conversion()    */
 #endif
 
-#if !defined(NV_FILE_OPERATIONS_HAS_IOCTL) && \
-  !defined(NV_FILE_OPERATIONS_HAS_UNLOCKED_IOCTL)
-#error "struct file_operations compile test likely failed!"
-#endif
-
 #if defined(CONFIG_VGA_ARB)
 #include <linux/vgaarb.h>
 #endif
@@ -182,13 +171,12 @@ static inline uid_t __kuid_val(kuid_t uid)
 #include <linux/swiotlb.h>
 #endif
 
-#if defined(NV_SCATTERLIST_HAS_PAGE_LINK)
 #include <linux/scatterlist.h>
-#endif
-
 #include <linux/completion.h>
 #include <linux/highmem.h>
 
+#include <linux/nodemask.h>
+
 #include <linux/workqueue.h>        /* workqueue                        */
 #include <nv-kthread-q.h>           /* kthread based queue              */
 
@@ -588,6 +576,17 @@ static inline void nv_iounmap(void *ptr, NvU64 size)
     iounmap(ptr);
 }
 
+static NvBool nv_numa_node_has_memory(int node_id)
+{
+    if (node_id < 0 || node_id >= MAX_NUMNODES)
+        return NV_FALSE;
+#if defined(NV_NODE_STATES_N_MEMORY_PRESENT)
+    return node_state(node_id, N_MEMORY) ? NV_TRUE : NV_FALSE;
+#else
+    return node_state(node_id, N_HIGH_MEMORY) ? NV_TRUE : NV_FALSE;
+#endif
+}
+
 #define NV_KMALLOC(ptr, size) \
     { \
         (ptr) = kmalloc(size, NV_GFP_KERNEL); \
@@ -1170,7 +1169,6 @@ nv_dma_maps_swiotlb(struct device *dev)
      * get_dma_ops() interface.
      */
     #if defined(NV_GET_DMA_OPS_PRESENT)
-      #if defined(NV_DMA_MAP_OPS_PRESENT) || defined(NV_SWIOTLB_DMA_OPS_PRESENT)
     /*
      * The __attribute__ ((unused)) is necessary because in at least one
      * case, *none* of the preprocessor branches below are taken, and
@@ -1179,13 +1177,10 @@ nv_dma_maps_swiotlb(struct device *dev)
      * case.
      */
     const struct dma_map_ops *ops __attribute__ ((unused)) = get_dma_ops(dev);
-      #else
-    const struct dma_mapping_ops *ops __attribute__ ((unused)) = get_dma_ops(dev);
-      #endif
     #else
     const struct dma_mapping_ops *ops __attribute__ ((unused)) = dma_ops;
     #endif
-    #if defined(NV_DMA_MAP_OPS_PRESENT) || defined(NV_SWIOTLB_DMA_OPS_PRESENT)
+
     /*
      * The switch from dma_mapping_ops -> dma_map_ops coincided with the
      * switch from swiotlb_map_sg -> swiotlb_map_sg_attrs.
@@ -1206,9 +1201,9 @@ nv_dma_maps_swiotlb(struct device *dev)
        * NV_IS_EXPORT_SYMBOL_PRESENT_swiotlb_dma_ops == 0) does
        * nothing, and ends up dropping us out to the last line of this function,
        * effectively returning false. The nearly-human-readable version of that
-       * case is "Either struct dma_map_ops or struct swiotlb_dma_ops is present
-       * (NV_DMA_MAP_OPS_PRESENT or NV_SWIOTLB_DMA_OPS_PRESENT is defined) but
-       * neither swiotlb_map_sg_attrs nor swiotlb_dma_ops is present".
+       * case is "struct swiotlb_dma_ops is present (NV_SWIOTLB_DMA_OPS_PRESENT
+       * is defined) but neither swiotlb_map_sg_attrs nor swiotlb_dma_ops is
+       * present".
        *
        * That can happen on kernels that fall within below range:
        *
@@ -1232,17 +1227,6 @@ nv_dma_maps_swiotlb(struct device *dev)
        * we just return NV_FALSE and in nv_compute_gfp_mask() we check for
        * whether swiotlb could possibly be used (outside of swiotlb=force).
        */
-    #else
-    swiotlb_in_use = (ops->map_sg == swiotlb_map_sg);
-    #endif
-  #elif defined(NVCPU_X86_64)
-    /*
-     * Fallback for old 2.6 kernels - if the DMA operations infrastructure
-     * isn't in place, use the swiotlb flag. Before dma_ops was added, this
-     * flag used to be exported. It still exists in modern kernels but is no
-     * longer exported.
-     */
-    swiotlb_in_use = (swiotlb == 1);
   #endif
 #endif
 
@@ -1285,7 +1269,7 @@ typedef struct nv_dma_map_s {
         } contig;
     } mapping;
 
-    struct pci_dev *pci_dev;
+    struct device *dev;
 } nv_dma_map_t;
 
 #define NV_FOR_EACH_DMA_SUBMAP(dm, sm, i)                                     \
@@ -1404,6 +1388,12 @@ typedef struct nv_linux_state_s {
         NvU32 displayId;
     } backlight;
 
+    /*
+     * file handle for pci sysfs config file (/sys/bus/pci/devices/.../config)
+     * which will be opened during device probe
+     */   
+    struct file *sysfs_config_file;
+
     /* Per-GPU queue */
     struct os_work_queue queue;
 
@@ -1413,6 +1403,9 @@ typedef struct nv_linux_state_s {
     NvBool all_mappings_revoked;
     NvBool safe_to_mmap;
     NvBool gpu_wakeup_callback_needed;
+
+    /* Per-device notifier block for ACPI events */
+    struct notifier_block acpi_nb;
 } nv_linux_state_t;
 
 extern nv_linux_state_t *nv_linux_devices;
@@ -1559,31 +1552,6 @@ typedef struct
 #define NV_ATOMIC_DEC(data)             atomic_dec(&(data))
 #define NV_ATOMIC_DEC_AND_TEST(data)    atomic_dec_and_test(&(data))
 
-#if defined(NV_ATOMIC_LONG_PRESENT)
-    typedef atomic_long_t nv_atomic_long_t;
-    #define NV_ATOMIC_LONG_READ(data)           atomic_long_read(&(data))
-    #define NV_ATOMIC_LONG_SET(data,val)        atomic_long_set(&(data), (val))
-    #define NV_ATOMIC_LONG_INC(data)            atomic_long_inc(&(data))
-    #define NV_ATOMIC_LONG_DEC(data)            atomic_long_dec(&(data))
-#else
-    /* atomic_long_t is not present, so we have to use the 32- or 64-bit native
-       type, which is guaranteed to be present */
-    #if BITS_PER_LONG == 64
-        typedef atomic64_t nv_atomic_long_t;
-        #define NV_ATOMIC_LONG_READ(data)       atomic64_read(&(data))
-        #define NV_ATOMIC_LONG_SET(data,val)    atomic64_set(&(data), (val))
-        #define NV_ATOMIC_LONG_INC(data)        atomic64_inc(&(data))
-        #define NV_ATOMIC_LONG_DEC(data)        atomic64_dec(&(data))
-    #else
-        /* BITS_PER_LONG != 64 */
-        typedef atomic_t nv_atomic_long_t;
-        #define NV_ATOMIC_LONG_READ(data)       atomic_read(&(data))
-        #define NV_ATOMIC_LONG_SET(data,val)    atomic_set(&(data), (val))
-        #define NV_ATOMIC_LONG_INC(data)        atomic_inc(&(data))
-        #define NV_ATOMIC_LONG_DEC(data)        atomic_dec(&(data))
-    #endif
-#endif /* NV_ATOMIC_LONG_PRESENT */
-
 
 
 
@@ -1611,9 +1579,12 @@ static inline NV_STATUS nv_check_gpu_state(nv_state_t *nv)
 }
 
 extern NvU32 NVreg_EnableUserNUMAManagement;
+extern NvU32 NVreg_RegisterPCIDriver;
 
 extern nv_pci_info_t nv_assign_gpu_pci_info[NV_MAX_DEVICES];
 extern NvU32 nv_assign_gpu_count;
+extern NvU32 num_probed_nv_devices;
+extern NvU32 num_nv_devices;
 
 #define NV_IS_ASSIGN_GPU_PCI_INFO_SPECIFIED()     \
     (!((nv_assign_gpu_pci_info[0].domain == 0) && \
@@ -1626,12 +1597,6 @@ extern NvU32 nv_assign_gpu_count;
 #define NV_FILE_INODE(file) (file)->f_dentry->d_inode
 #endif
 
-/* Stub out UVM in multi-RM builds */
-
-#if (NV_BUILD_MODULE_INSTANCES != 0)
-#undef NV_UVM_ENABLE
-#endif
-
 #if defined(NV_DOM0_KERNEL_PRESENT) || defined(NV_VGPU_KVM_BUILD)
 #define NV_VGX_HYPER
 #if defined(NV_XEN_IOEMU_INJECT_MSI)
@@ -1639,12 +1604,6 @@ extern NvU32 nv_assign_gpu_count;
 #endif
 #endif
 
-#if defined(NV_FOR_EACH_ONLINE_NODE_PRESENT)
-#define NV_FOR_EACH_ONLINE_NODE(nid)   for_each_online_node(nid)
-#else
-#define NV_FOR_EACH_ONLINE_NODE(nid)   for (nid = 0; nid < MAX_NUMNODES; nid++)
-#endif
-
 static inline NvU64 nv_node_end_pfn(int nid)
 {
 #if defined(NV_NODE_END_PFN_PRESENT)
@@ -1785,4 +1744,9 @@ static inline NvU32 nv_default_irq_flags(nv_state_t *nv)
 #include <linux/backlight.h>
 #endif
 
+#define MODULE_BASE_NAME "nvidia"
+#define MODULE_INSTANCE_NUMBER 0
+#define MODULE_INSTANCE_STRING ""
+#define MODULE_NAME MODULE_BASE_NAME MODULE_INSTANCE_STRING
+
 #endif  /* _NV_LINUX_H_ */
