@@ -508,14 +508,7 @@ typedef struct
     //
     // Cause that triggered the migration
     //
-    // TODO: Bug 1766109: uvm8: delete UVM-Lite files and remove -lite mode
-    // TODO: Bug 1766104: uvm8: delete uvmfull/ subdirectory before release
-    // Remove direction when -lite and -full code is removed
-    union
-    {
-        NvU8 migrationCause;
-        NvU8 direction;
-    };
+    NvU8 migrationCause;
     //
     // Indices are used for the source and destination of migration instead of
     // using gpu uuid/cpu id. This reduces the size of each event. The index to
