@@ -217,7 +217,7 @@ struct uvm_channel_manager_struct
         // If an entry is invalid, the value is UVM_COPY_ENGINE_COUNT_MAX, and
         // the default Copy Engine (default_for_type[UVM_CHANNEL_GPU_TO_GPU])
         // must be used instead.
-        NvU32 gpu_to_gpu[UVM_MAX_GPUS];
+        NvU32 gpu_to_gpu[UVM_ID_MAX_GPUS];
     } ce_to_use;
 
     struct
