@@ -61,7 +61,9 @@ typedef struct uvm_make_resident_context_struct uvm_make_resident_context_t;
 
 typedef struct uvm_gpu_va_space_struct uvm_gpu_va_space_t;
 
+typedef struct uvm_thread_context_lock_struct uvm_thread_context_lock_t;
 typedef struct uvm_thread_context_struct uvm_thread_context_t;
+typedef struct uvm_thread_context_wrapper_struct uvm_thread_context_wrapper_t;
 
 typedef struct uvm_perf_module_struct uvm_perf_module_t;
 
