@@ -1,5 +1,5 @@
 /*******************************************************************************
-    Copyright (c) 2016 NVIDIA Corporation
+    Copyright (c) 2016-2019 NVIDIA Corporation
 
     Permission is hereby granted, free of charge, to any person obtaining a copy
     of this software and associated documentation files (the "Software"), to
@@ -21,12 +21,14 @@
 
 *******************************************************************************/
 
+#include "uvm8_api.h"
 #include "uvm8_global.h"
 #include "uvm8_gpu_isr.h"
 #include "uvm8_hal.h"
 #include "uvm8_gpu.h"
 #include "uvm8_gpu_access_counters.h"
 #include "uvm8_gpu_non_replayable_faults.h"
+#include "uvm8_thread_context.h"
 
 // Level-based vs pulse-based interrupts
 // =====================================
@@ -51,15 +53,15 @@
 
 // For use by the nv_kthread_q that is servicing the replayable fault bottom
 // half, only.
-static void replayable_faults_isr_bottom_half(void *args);
+static void replayable_faults_isr_bottom_half_entry(void *args);
 
 // For use by the nv_kthread_q that is servicing the replayable fault bottom
 // half, only.
-static void non_replayable_faults_isr_bottom_half(void *args);
+static void non_replayable_faults_isr_bottom_half_entry(void *args);
 
 // For use by the nv_kthread_q that is servicing the replayable fault bottom
 // half, only.
-static void access_counters_isr_bottom_half(void *args);
+static void access_counters_isr_bottom_half_entry(void *args);
 
 // Increments the reference count tracking whether replayable page fault
 // interrupts should be disabled. The caller is guaranteed that replayable page
@@ -191,7 +193,7 @@ static unsigned schedule_access_counters_handler(uvm_gpu_t *gpu)
 //         available for the future. RM's interrupt handling tends to evolve as new chips and
 //         new interrupts get created.
 
-NV_STATUS uvm8_isr_top_half(const NvProcessorUuid *gpu_uuid)
+static NV_STATUS uvm8_isr_top_half(const NvProcessorUuid *gpu_uuid)
 {
     uvm_gpu_t *gpu;
     unsigned num_handlers_scheduled = 0;
@@ -257,6 +259,29 @@ NV_STATUS uvm8_isr_top_half(const NvProcessorUuid *gpu_uuid)
     return status;
 }
 
+NV_STATUS uvm8_isr_top_half_entry(const NvProcessorUuid *gpu_uuid)
+{
+    UVM_ENTRY_RET(uvm8_isr_top_half(gpu_uuid));
+}
+
+static NV_STATUS init_queue_on_node(nv_kthread_q_t *queue, const char *name, int node)
+{
+    NV_STATUS status;
+
+    if (UVM_THREAD_AFFINITY_SUPPORTED() && node != -1 && !cpumask_empty(uvm_cpumask_of_node(node))) {
+        status = errno_to_nv_status(nv_kthread_q_init_on_node(queue, name, node));
+        if (status != NV_OK)
+            return status;
+
+        status = errno_to_nv_status(set_cpus_allowed_ptr(queue->q_kthread, uvm_cpumask_of_node(node)));
+    }
+    else {
+        status = errno_to_nv_status(nv_kthread_q_init(queue, name));
+    }
+
+    return status;
+}
+
 NV_STATUS uvm_gpu_init_isr(uvm_gpu_t *gpu)
 {
     NV_STATUS status = NV_OK;
@@ -269,13 +294,19 @@ NV_STATUS uvm_gpu_init_isr(uvm_gpu_t *gpu)
             return status;
         }
 
-        nv_kthread_q_item_init(&gpu->isr.replayable_faults.bottom_half_q_item, replayable_faults_isr_bottom_half, gpu);
+        nv_kthread_q_item_init(&gpu->isr.replayable_faults.bottom_half_q_item,
+                               replayable_faults_isr_bottom_half_entry,
+                               gpu);
+
+        gpu->isr.replayable_faults.stats.cpu_exec_count =
+            uvm_kvmalloc_zero(sizeof(*gpu->isr.replayable_faults.stats.cpu_exec_count) * num_possible_cpus());
+        if (!gpu->isr.replayable_faults.stats.cpu_exec_count)
+            return NV_ERR_NO_MEMORY;
 
         gpu->isr.replayable_faults.handling = true;
 
-        snprintf(kthread_name, sizeof(kthread_name), "UVM GPU%u BH", gpu->id);
-        status = errno_to_nv_status(nv_kthread_q_init(&gpu->isr.bottom_half_q, kthread_name));
-
+        snprintf(kthread_name, sizeof(kthread_name), "UVM GPU%u BH", uvm_id_value(gpu->id));
+        status = init_queue_on_node(&gpu->isr.bottom_half_q, kthread_name, gpu->closest_cpu_numa_node);
         if (status != NV_OK) {
             UVM_ERR_PRINT("Failed in nv_kthread_q_init for bottom_half_q: %s, GPU %s\n",
                           nvstatusToString(status),
@@ -285,14 +316,18 @@ NV_STATUS uvm_gpu_init_isr(uvm_gpu_t *gpu)
 
         if (gpu->non_replayable_faults_supported) {
             nv_kthread_q_item_init(&gpu->isr.non_replayable_faults.bottom_half_q_item,
-                                   non_replayable_faults_isr_bottom_half,
+                                   non_replayable_faults_isr_bottom_half_entry,
                                    gpu);
 
+            gpu->isr.non_replayable_faults.stats.cpu_exec_count =
+                uvm_kvmalloc_zero(sizeof(*gpu->isr.non_replayable_faults.stats.cpu_exec_count) * num_possible_cpus());
+            if (!gpu->isr.non_replayable_faults.stats.cpu_exec_count)
+                return NV_ERR_NO_MEMORY;
+
             gpu->isr.non_replayable_faults.handling = true;
 
-            snprintf(kthread_name, sizeof(kthread_name), "UVM GPU%u KC", gpu->id);
-            status = errno_to_nv_status(nv_kthread_q_init(&gpu->isr.kill_channel_q, kthread_name));
-
+            snprintf(kthread_name, sizeof(kthread_name), "UVM GPU%u KC", uvm_id_value(gpu->id));
+            status = init_queue_on_node(&gpu->isr.kill_channel_q, kthread_name, gpu->closest_cpu_numa_node);
             if (status != NV_OK) {
                 UVM_ERR_PRINT("Failed in nv_kthread_q_init for kill_channel_q: %s, GPU %s\n",
                               nvstatusToString(status),
@@ -310,11 +345,17 @@ NV_STATUS uvm_gpu_init_isr(uvm_gpu_t *gpu)
                 return status;
             }
 
-            nv_kthread_q_item_init(&gpu->isr.access_counters.bottom_half_q_item, access_counters_isr_bottom_half, gpu);
+            nv_kthread_q_item_init(&gpu->isr.access_counters.bottom_half_q_item,
+                                   access_counters_isr_bottom_half_entry,
+                                   gpu);
 
             // Access counters interrupts are initially disabled. They are
             // dynamically enabled when the GPU is registered on a VA space.
             gpu->isr.access_counters.handling_ref_count = 0;
+            gpu->isr.access_counters.stats.cpu_exec_count =
+                uvm_kvmalloc_zero(sizeof(*gpu->isr.access_counters.stats.cpu_exec_count) * num_possible_cpus());
+            if (!gpu->isr.access_counters.stats.cpu_exec_count)
+                return NV_ERR_NO_MEMORY;
         }
     }
 
@@ -323,7 +364,7 @@ NV_STATUS uvm_gpu_init_isr(uvm_gpu_t *gpu)
 
 void uvm_gpu_disable_isr(uvm_gpu_t *gpu)
 {
-    UVM_ASSERT(g_uvm_global.gpus[uvm_gpu_index(gpu->id)] != gpu);
+    UVM_ASSERT(g_uvm_global.gpus[uvm_id_gpu_index(gpu->id)] != gpu);
     UVM_ASSERT(gpu->isr.access_counters.handling_ref_count == 0);
 
     // Now that the GPU is safely out of the global table, lock the GPU and mark
@@ -382,18 +423,27 @@ void uvm_gpu_deinit_isr(uvm_gpu_t *gpu)
         // successfully initialized.
         uvm_gpu_deinit_access_counters(gpu);
     }
+
+    uvm_kvfree(gpu->isr.replayable_faults.stats.cpu_exec_count);
+    uvm_kvfree(gpu->isr.non_replayable_faults.stats.cpu_exec_count);
+    uvm_kvfree(gpu->isr.access_counters.stats.cpu_exec_count);
 }
 
 static void replayable_faults_isr_bottom_half(void *args)
 {
     uvm_gpu_t *gpu = (uvm_gpu_t *)args;
+    unsigned int cpu;
 
     UVM_ASSERT(gpu->replayable_faults_supported);
 
     // Multiple bottom halves for replayable faults can be running
     // concurrently, but only one can be running this function for a given GPU
     // since we enter with the replayable_faults.service_lock held.
-    ++gpu->isr.replayable_faults.bottom_half_count;
+    cpu = get_cpu();
+    ++gpu->isr.replayable_faults.stats.bottom_half_count;
+    cpumask_set_cpu(cpu, &gpu->isr.replayable_faults.stats.cpus_used_mask);
+    ++gpu->isr.replayable_faults.stats.cpu_exec_count[cpu];
+    put_cpu();
 
     uvm_gpu_service_replayable_faults(gpu);
 
@@ -401,9 +451,15 @@ static void replayable_faults_isr_bottom_half(void *args)
     uvm_gpu_kref_put(gpu);
 }
 
+static void replayable_faults_isr_bottom_half_entry(void *args)
+{
+   UVM_ENTRY_VOID(replayable_faults_isr_bottom_half(args));
+}
+
 static void non_replayable_faults_isr_bottom_half(void *args)
 {
     uvm_gpu_t *gpu = (uvm_gpu_t *)args;
+    unsigned int cpu;
 
     UVM_ASSERT(gpu->non_replayable_faults_supported);
 
@@ -412,7 +468,11 @@ static void non_replayable_faults_isr_bottom_half(void *args)
     // Multiple bottom halves for non-replayable faults can be running
     // concurrently, but only one can enter this section for a given GPU
     // since we acquired the non_replayable_faults.service_lock
-    ++gpu->isr.non_replayable_faults.bottom_half_count;
+    cpu = get_cpu();
+    ++gpu->isr.non_replayable_faults.stats.bottom_half_count;
+    cpumask_set_cpu(cpu, &gpu->isr.non_replayable_faults.stats.cpus_used_mask);
+    ++gpu->isr.non_replayable_faults.stats.cpu_exec_count[cpu];
+    put_cpu();
 
     uvm_gpu_service_non_replayable_fault_buffer(gpu);
 
@@ -420,16 +480,26 @@ static void non_replayable_faults_isr_bottom_half(void *args)
     uvm_gpu_kref_put(gpu);
 }
 
+static void non_replayable_faults_isr_bottom_half_entry(void *args)
+{
+   UVM_ENTRY_VOID(non_replayable_faults_isr_bottom_half(args));
+}
+
 static void access_counters_isr_bottom_half(void *args)
 {
     uvm_gpu_t *gpu = (uvm_gpu_t *)args;
+    unsigned int cpu;
 
     UVM_ASSERT(gpu->access_counters_supported);
 
     // Multiple bottom halves for counter notifications can be running
     // concurrently, but only one can be running this function for a given GPU
     // since we enter with the access_counters_isr_lock held.
-    ++gpu->isr.access_counters.bottom_half_count;
+    cpu = get_cpu();
+    ++gpu->isr.access_counters.stats.bottom_half_count;
+    cpumask_set_cpu(cpu, &gpu->isr.access_counters.stats.cpus_used_mask);
+    ++gpu->isr.access_counters.stats.cpu_exec_count[cpu];
+    put_cpu();
 
     uvm_gpu_service_access_counters(gpu);
 
@@ -437,6 +507,11 @@ static void access_counters_isr_bottom_half(void *args)
     uvm_gpu_kref_put(gpu);
 }
 
+static void access_counters_isr_bottom_half_entry(void *args)
+{
+   UVM_ENTRY_VOID(access_counters_isr_bottom_half(args));
+}
+
 void uvm_gpu_replayable_faults_isr_lock(uvm_gpu_t *gpu)
 {
     UVM_ASSERT(uvm_gpu_retained_count(gpu) > 0);
