@@ -1,5 +1,5 @@
 /*******************************************************************************
-    Copyright (c) 2017 NVIDIA Corporation
+    Copyright (c) 2017-2019 NVIDIA Corporation
 
     Permission is hereby granted, free of charge, to any person obtaining a copy
     of this software and associated documentation files (the "Software"), to
@@ -22,6 +22,7 @@
 
 #include "nv_uvm_interface.h"
 #include "uvm_common.h"
+#include "uvm8_api.h"
 #include "uvm8_gpu_non_replayable_faults.h"
 #include "uvm8_gpu.h"
 #include "uvm8_hal.h"
@@ -276,7 +277,7 @@ static NV_STATUS service_managed_fault_in_block_locked(uvm_gpu_t *gpu,
     }
 
     // Check logical permissions
-    status = uvm_va_range_check_logical_permissions(va_block->va_range,
+    status = uvm_va_range_check_logical_permissions(va_range,
                                                     gpu->id,
                                                     fault_entry->fault_access_type,
                                                     uvm_range_group_address_migratable(va_space,
@@ -311,8 +312,8 @@ static NV_STATUS service_managed_fault_in_block_locked(uvm_gpu_t *gpu,
     uvm_processor_mask_set(&service_context->resident_processors, new_residency);
 
     // The masks need to be fully zeroed as the fault region may grow due to prefetching
-    uvm_page_mask_zero(&service_context->per_processor_masks[new_residency].new_residency);
-    uvm_page_mask_set(&service_context->per_processor_masks[new_residency].new_residency, page_index);
+    uvm_page_mask_zero(&service_context->per_processor_masks[uvm_id_value(new_residency)].new_residency);
+    uvm_page_mask_set(&service_context->per_processor_masks[uvm_id_value(new_residency)].new_residency, page_index);
 
     if (read_duplicate) {
         uvm_page_mask_zero(&service_context->read_duplicate_mask);
@@ -381,6 +382,11 @@ static void kill_channel_delayed(void *_user_channel)
     uvm_user_channel_release(user_channel);
 }
 
+static void kill_channel_delayed_entry(void *user_channel)
+{
+    UVM_ENTRY_VOID(kill_channel_delayed(user_channel));
+}
+
 static void schedule_kill_channel(uvm_gpu_t *gpu,
                                   uvm_fault_buffer_entry_t *fault_entry,
                                   uvm_user_channel_t *user_channel)
@@ -409,7 +415,7 @@ static void schedule_kill_channel(uvm_gpu_t *gpu,
 
     // Schedule a work item to kill the channel
     nv_kthread_q_item_init(&user_channel->kill_channel.kill_channel_q_item,
-                           kill_channel_delayed,
+                           kill_channel_delayed_entry,
                            user_channel);
 
     nv_kthread_q_schedule_q_item(&gpu->isr.kill_channel_q,
