@@ -1,5 +1,5 @@
 /*******************************************************************************
-    Copyright (c) 2015-2018 NVIDIA Corporation
+    Copyright (c) 2015-2019 NVIDIA Corporation
 
     Permission is hereby granted, free of charge, to any person obtaining a copy
     of this software and associated documentation files (the "Software"), to
@@ -30,6 +30,7 @@
 #include "uvm8_hal_types.h"
 #include "uvm8_push.h"
 #include "uvm8_gpu.h"
+#include "uvm8_test_ioctl.h"
 
 typedef void (*uvm_hal_init_t)(uvm_push_t *push);
 void uvm_hal_kepler_ce_init(uvm_push_t *push);
@@ -56,7 +57,7 @@ typedef void (*uvm_hal_noop_t)(uvm_push_t *push, NvU32 size);
 void uvm_hal_kepler_host_noop(uvm_push_t *push, NvU32 size);
 
 // Host-generated interrupt method. This will generate a call to
-// uvm8_isr_top_half.
+// uvm8_isr_top_half_entry.
 //
 // This is a non-stalling interrupt, which means that it's fire-and-forget. Host
 // will not stall method processing nor stop channel switching, which means that
