@@ -1,5 +1,5 @@
 /*******************************************************************************
-    Copyright (c) 2015-2018 NVIDIA Corporation
+    Copyright (c) 2015-2019 NVIDIA Corporation
 
     Permission is hereby granted, free of charge, to any person obtaining a copy
     of this software and associated documentation files (the "Software"), to
@@ -224,7 +224,7 @@ struct uvm_gpu_chunk_struct
         // This allows determining what PMM owns the chunk. Users of this field
         // must only use it if the owning GPU is retained.
         // TODO: Bug 2008200: Enforce single PMM instance per GPU
-        uvm_gpu_id_t                          gpu_id : order_base_2(UVM_MAX_PROCESSORS);
+        NvU32                           gpu_id_index : order_base_2(UVM_GLOBAL_ID_MAX_PROCESSORS);
     };
 
     // List entry.
@@ -263,9 +263,42 @@ typedef struct uvm_gpu_root_chunk_struct
     // uvm_pmm_gpu_t::root_chunks.indirect_peer[id].dma_addrs.
     //
     // Protected by the corresponding root chunk bit lock.
+
+
+
+
+
     uvm_processor_mask_t indirect_peers_mapped;
 } uvm_gpu_root_chunk_t;
 
+typedef struct
+{
+    // Indirect peers are GPUs which can coherently access this GPU's memory,
+    // but are routed through an intermediate processor. Indirect peers access
+    // each others' memory with the SYS aperture rather then a PEER aperture,
+    // meaning they need IOMMU mappings:
+    //
+    // accessing_gpu ==> IOMMU ==> CPU ==> owning_gpu (this GPU)
+    //
+    // This array has one entry per root chunk on this GPU. Each entry
+    // contains the IOMMU address accessing_gpu needs to use in order to
+    // access this GPU's root chunk. The root chunks are mapped as whole
+    // regions both for tracking simplicity and to allow GPUs to map with
+    // large PTEs.
+    //
+    // An array entry is valid iff accessing_gpu's ID is set in the
+    // corresponding root chunk's indirect_peers_mapped mask.
+    //
+    // Management of these addresses would be simpler if they were stored
+    // in the root chunks themselves, but in the common case there are only
+    // a small number of indirect peers in a system. Dynamic array
+    // allocation per indirect peer wastes less memory.
+    NvU64 *dma_addrs;
+
+    // Number of this GPU's root chunks mapped for each indirect peer.
+    atomic64_t map_count;
+} uvm_gpu_root_chunk_indirect_peer_t;
+
 typedef struct
 {
     // TODO: Bug 2008200: Remove this field and use container_of
@@ -277,6 +310,9 @@ typedef struct
     // PMA (Physical Memory Allocator) opaque handle
     void *pma;
 
+    // PMA statistics used for eviction heuristics
+    const UvmPmaStatistics *pma_stats;
+
     struct
     {
         // Array of all root chunks indexed by their physical address divided by
@@ -305,33 +341,7 @@ typedef struct
         // List of root chunks used by VA blocks
         struct list_head va_block_used;
 
-        struct
-        {
-            // Indirect peers are GPUs which can coherently access this GPU's
-            // memory, but are routed through an intermediate processor.
-            // Indirect peers access each others' memory with the SYS aperture
-            // rather then a PEER aperture, meaning they need IOMMU mappings:
-            //
-            // accessing_gpu ==> IOMMU ==> CPU ==> owning_gpu (this GPU)
-            //
-            // This array has one entry per root chunk on this GPU. Each entry
-            // contains the IOMMU address accessing_gpu needs to use in order to
-            // access this GPU's root chunk. The root chunks are mapped as whole
-            // regions both for tracking simplicity and to allow GPUs to map
-            // with large PTEs.
-            //
-            // An array entry is valid iff accessing_gpu's ID is set in the
-            // corresponding root chunk's indirect_peers_mapped mask.
-            //
-            // Management of these addresses would be simpler if they were
-            // stored in the root chunks themselves, but in the common case
-            // there are only a small number of indirect peers in a system.
-            // Dynamic array allocation per indirect peer wastes less memory.
-            NvU64 *dma_addrs;
-
-            // Number of this GPU's root chunks mapped for each indirect peer.
-            atomic64_t map_count;
-        } indirect_peer[UVM_MAX_GPUS];
+        uvm_gpu_root_chunk_indirect_peer_t indirect_peer[UVM_ID_MAX_GPUS];
     } root_chunks;
 
     // Lock protecting PMA allocation, freeing and eviction
