@@ -34,8 +34,8 @@
 #include "uvm8_gpu.h"
 #include "uvm8_tracker.h"
 #include "uvm8_map_external.h"
-#include "uvm8_init.h"
 #include "nv_uvm_interface.h"
+#include "uvm8_test.h"
 
 #include <linux/sort.h>
 
@@ -193,7 +193,7 @@ static NV_STATUS uvm_user_channel_create(uvm_va_space_t *va_space,
     }
 
     if (user_channel->tsg.valid)
-        UVM_ASSERT(user_channel->tsg.max_subctx_count <= gpu->max_subcontexts);
+        UVM_ASSERT(user_channel->tsg.max_subctx_count <= gpu->rm_info.maxSubctxCount);
 
     // If num_resources == 0, as can happen with CE channels, we ignore base and
     // length.
